import React from 'react';
import PropTypes from 'prop-types';
import Animated from 'animated/lib/targets/react-dom';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

function _assertThisInitialized(self) {
  if (self === void 0) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return self;
}

function createAnimatedValue(name, from, to) {
  var animation = new Animated.Value(0);
  var result = {
    name: name,
    animation: animation,
    interpolate: animation.interpolate({
      inputRange: [0, 1],
      outputRange: [from, to]
    })
  };
  return result;
}

var Spring =
/*#__PURE__*/
function (_React$PureComponent) {
  _inheritsLoose(Spring, _React$PureComponent);

  function Spring(props) {
    var _this;

    _this = _React$PureComponent.call(this) || this;

    _initialiseProps.call(_assertThisInitialized(_this));

    var children = props.children,
        to = props.to,
        from = props.from;
    _this.component = Animated.createAnimatedComponent(children);
    _this.animations = Object.entries(to).map(function (_ref) {
      var key = _ref[0],
          value = _ref[1];
      return createAnimatedValue(key, from[key] !== undefined ? from[key] : value, value);
    });
    _this.to = _this.animations.reduce(function (acc, anim) {
      var _extends2;

      return _extends({}, acc, (_extends2 = {}, _extends2[anim.name] = anim.interpolate, _extends2));
    }, {});
    return _this;
  }

  var _proto = Spring.prototype;

  _proto.componentWillReceiveProps = function componentWillReceiveProps(props) {
    this.update(props);
  };

  _proto.componentWillMount = function componentWillMount() {
    this.animations.forEach(function (anim) {
      return Animated.spring(anim.animation, {
        toValue: 1
      }).start();
    });
  };

  _proto.render = function render() {
    return React.createElement(this.component, this.to);
  };

  return Spring;
}(React.PureComponent);

Spring.defaultProps = {
  to: {},
  from: {}
};

var _initialiseProps = function _initialiseProps() {
  var _this2 = this;

  this.update = function (props) {
    var to = props.to,
        from = props.from;

    for (var _iterator = _this2.animations, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
      var _ref2;

      if (_isArray) {
        if (_i >= _iterator.length) break;
        _ref2 = _iterator[_i++];
      } else {
        _i = _iterator.next();
        if (_i.done) break;
        _ref2 = _i.value;
      }

      var _anim = _ref2;
      var currentValue = _anim.animation._value;

      _anim.animation.stopAnimation();

      if (to[_anim.name] !== _anim.to) {
        _anim.animation = new Animated.Value(0);
        _anim.interpolate = _anim.animation.interpolate({
          inputRange: [0, 1],
          outputRange: [_anim.interpolate._interpolation(currentValue), to[_anim.name]]
        });
        Animated.spring(_anim.animation, {
          toValue: 1
        }).start();
      }
    }

    _this2.to = _this2.animations.reduce(function (acc, anim) {
      var _extends3;

      return _extends({}, acc, (_extends3 = {}, _extends3[anim.name] = anim.interpolate, _extends3));
    }, {});
  };
};
process.env.NODE_ENV !== "production" ? Spring.propTypes = {
  to: PropTypes.object,
  from: PropTypes.object
} : void 0;

export default Spring;
